import * as React from "react";
import { useMemo, useState, useEffect, useRef } from "react";
import { uid } from "uid";
import { debounce } from "lodash-es";
import ResizeObserver from "resize-observer-polyfill";
import { defaultProps } from "../react-desc-prop-types.js";
import { useScales } from "./useScales.js";
import { stackData } from "../helpers/index.js";
import { useInternalMargins } from "./useInternalMargins.js";
import { useGetters } from "./useGetters.js";
import { useValidateProps } from "./useValidateProps.js";
import { COLOR_PALLET } from "../helpers/colorPallet.js";
const useResizeObserver = (ref) => {
  const [dimensions, setDimensions] = useState(0);
  useEffect(() => {
    if (ref) {
      const observeTarget = ref;
      const resizeObserver = new ResizeObserver(
        debounce((entries) => {
          entries.forEach((entry) => {
            setDimensions(Math.ceil(entry.contentRect.width));
          });
        }, 200)
      );
      resizeObserver.observe(observeTarget);
      return () => {
        resizeObserver.unobserve(observeTarget);
      };
    }
    return () => {
    };
  }, [ref]);
  return dimensions;
};
const useChart = (props) => {
  useValidateProps(props);
  const { series, yAxis, xAxis, width: userWidth, height: userHeight, groups: groupsStacked } = props;
  const userRatio = useMemo(
    () => props?.xAxis.advanced?.pointSpacing?.value ?? props?.yAxis.advanced?.pointSpacing?.value ?? 1,
    [props?.xAxis.advanced?.pointSpacing?.value, props?.yAxis.advanced?.pointSpacing?.value]
  );
  const originalSeries = useMemo(() => {
    if (series.length === 0) {
      return defaultProps.series;
    }
    return series;
  }, [series]);
  const [containerRef, setContainerRef] = useState(null);
  const dimensionWidth = useResizeObserver(containerRef);
  const width = useMemo(() => userWidth ?? dimensionWidth ?? 0, [dimensionWidth, userWidth]);
  const height = useMemo(() => userHeight ?? 500, [userHeight]);
  const [isGrabbed, setIsGrabbed] = useState(false);
  const [startPosition, setStartPosition] = useState(0);
  const [isZooming, setIsZooming] = useState(false);
  const [zoomStartingPosition, setZoomStartingPosition] = useState(0);
  const [movingPosition, setMovingPosition] = useState(0);
  const [activePoint, setActivePoint] = useState(null);
  const [activeSerie, setActiveSerie] = useState("");
  const [scrapperPosY, setScrapperPosY] = useState(-1);
  const [scrapperPosX, setScrapperPosX] = useState(-1);
  const [xScrollbarPosition, setXScrollbarPosition] = useState(0);
  const [isScrollbarVisible, setIsScrollbarVisible] = useState(false);
  const [containerRatio, setContainerRatio] = useState(userRatio);
  const lastScrollbarPosition = useRef(0);
  const [axisLeftRef, setAxisLeftRef] = useState(null);
  const [axisBottomRef, setAxisBottomRef] = useState(null);
  const [axisRightRef, setAxisRightRef] = useState(null);
  const [leftLegend, setLeftLegend] = useState(null);
  const [rightLegend, setRightLegend] = useState(null);
  const [topLegend, setTopLegend] = useState(null);
  const [bottomLegend, setBottomLegend] = useState(null);
  const [rightLabel, setRightLabel] = useState(null);
  const [leftLabel, setLeftLabel] = useState(null);
  const [bottomLabel, setBottomLabel] = useState(null);
  const [toolbarRef, setToolbarRef] = useState(null);
  useEffect(() => {
    lastScrollbarPosition.current = 0;
    setXScrollbarPosition(0);
  }, [dimensionWidth]);
  useEffect(() => {
    setContainerRatio(userRatio);
  }, [userRatio]);
  const {
    toolbarHeight,
    axisBottomHeight,
    axisRightWidth,
    bottomLabelHeight,
    internalMargin,
    leftLegendWidth,
    rightLabelWidth,
    leftLabelHeight,
    rightLegendHeight,
    bottomLegendWidth,
    leftLabelWidth,
    axisLeftWidth,
    topLegendHeight,
    leftLegendHeight
  } = useInternalMargins({
    axisLeftRef,
    axisBottomRef,
    axisRightRef,
    leftLabel,
    leftLegend,
    rightLegend,
    topLegend,
    bottomLegend,
    rightLabel,
    bottomLabel,
    toolbarRef
  });
  const innerWidth = useMemo(
    () => width - internalMargin.left - internalMargin.right,
    [internalMargin.left, internalMargin.right, width]
  );
  const innerHeight = useMemo(
    () => height - internalMargin.top - internalMargin.bottom,
    [internalMargin.top, internalMargin.bottom, height]
  );
  const [hiddenSeries, setHiddenSeries] = useState([]);
  const defaultScale = useMemo(() => {
    if (yAxis.type === "band") {
      return "x";
    }
    if ([void 0, "band"].includes(xAxis.type)) {
      return "y";
    }
    return "y";
  }, [xAxis.type, yAxis.type]);
  const [currentDataWithNullValues, currentData] = useMemo(() => {
    const baseData = originalSeries.filter((serie) => !hiddenSeries.includes(serie.name)).map((d, i) => ({
      ...d,
      // we set a default on the scale if the user do not pass it
      scale: d.scale || defaultScale,
      key: `${d.name}-${i}`,
      data: d.data.map((value, index) => ({
        key: `${d.name}-${i}-${index}`,
        value,
        position: index,
        serie: d.name,
        scale: d.scale,
        color: COLOR_PALLET[d.color] ?? d.color
      }))
    }));
    const currentDataWithoutNulls = baseData.map((data) => ({
      ...data,
      data: data.data.filter((datum) => datum.value !== null && datum.value !== void 0)
    }));
    return [baseData, currentDataWithoutNulls];
  }, [defaultScale, hiddenSeries, originalSeries]);
  const isHorizontal = useMemo(() => yAxis?.type === "band", [yAxis]);
  const { stackedData, scalesStackedDataMap } = useMemo(
    () => stackData(groupsStacked, originalSeries, hiddenSeries, isHorizontal),
    [groupsStacked, originalSeries, hiddenSeries, isHorizontal]
  );
  const groups = useMemo(() => currentData.map((serie) => serie.name), [currentData]);
  const { xScale, yScale, y2Scale, subGroupScale, colorScale, getBandwidth } = useScales({
    props,
    originalSeries,
    innerHeight,
    innerWidth,
    groups,
    stackedData,
    scalesStackedDataMap,
    currentData,
    containerRatio
  });
  const {
    getXValue,
    getY2ScaleValue,
    getYValue,
    getXScaleValue,
    getYScaleValue,
    getYValueFormatted,
    getXValueFormatted
  } = useGetters({
    xAxis,
    yAxis,
    isHorizontal,
    xScale,
    yScale,
    y2Scale
  });
  const chartId = useMemo(() => uid(6), []);
  return useMemo(
    () => ({
      props,
      originalSeries,
      isZooming,
      setIsZooming,
      zoomStartingPosition,
      setZoomStartingPosition,
      setMovingPosition,
      movingPosition,
      innerHeight,
      innerWidth,
      groups,
      currentData,
      colorScale,
      stackedData,
      subGroupScale,
      xScale,
      yScale,
      y2Scale,
      containerRef,
      getBandwidth,
      scrapperPosY,
      setScrapperPosY,
      scrapperPosX,
      setScrapperPosX,
      activePoint,
      setActivePoint,
      activeSerie,
      setActiveSerie,
      xScrollbarPosition,
      setXScrollbarPosition,
      isGrabbed,
      setIsGrabbed,
      setContainerRef,
      startPosition,
      setStartPosition,
      isScrollbarVisible,
      setIsScrollbarVisible,
      lastScrollbarPosition,
      containerRatio,
      internalMargin,
      axisLeftRef,
      leftLegendWidth,
      leftLegend,
      setAxisLeftRef,
      setLeftLegend,
      setRightLegend,
      rightLegend,
      setTopLegend,
      setBottomLegend,
      setAxisBottomRef,
      axisBottomHeight,
      setAxisRightRef,
      setRightLabel,
      setBottomLabel,
      getXScaleValue,
      getYScaleValue,
      getY2ScaleValue,
      getYValueFormatted,
      getXValueFormatted,
      bottomLabelHeight,
      rightLabelWidth,
      axisRightWidth,
      getXValue,
      getYValue,
      isHorizontal,
      setLeftLabel,
      leftLabel,
      chartId,
      hiddenSeries,
      setHiddenSeries,
      height,
      width,
      setContainerRatio,
      toolbarHeight,
      setToolbarRef,
      leftLabelHeight,
      rightLegendHeight,
      bottomLegendWidth,
      bottomLegend,
      leftLabelWidth,
      axisLeftWidth,
      topLegendHeight,
      leftLegendHeight,
      currentDataWithNullValues
    }),
    [
      props,
      originalSeries,
      isZooming,
      zoomStartingPosition,
      movingPosition,
      innerHeight,
      innerWidth,
      groups,
      currentData,
      colorScale,
      stackedData,
      subGroupScale,
      xScale,
      yScale,
      y2Scale,
      containerRef,
      getBandwidth,
      scrapperPosY,
      getY2ScaleValue,
      scrapperPosX,
      activePoint,
      activeSerie,
      xScrollbarPosition,
      isGrabbed,
      startPosition,
      isScrollbarVisible,
      containerRatio,
      internalMargin,
      axisLeftRef,
      leftLegendWidth,
      leftLegend,
      rightLegend,
      axisBottomHeight,
      getXScaleValue,
      getYScaleValue,
      getYValueFormatted,
      getXValueFormatted,
      bottomLabelHeight,
      rightLabelWidth,
      axisRightWidth,
      getXValue,
      getYValue,
      isHorizontal,
      leftLabel,
      chartId,
      hiddenSeries,
      height,
      width,
      toolbarHeight,
      leftLabelHeight,
      rightLegendHeight,
      bottomLegendWidth,
      bottomLegend,
      leftLabelWidth,
      axisLeftWidth,
      topLegendHeight,
      leftLegendHeight,
      currentDataWithNullValues
    ]
  );
};
export {
  useChart
};
//# sourceMappingURL=useChart.js.map
