import * as React from "react";
import { useMemo, useCallback } from "react";
import { max, min, scaleLinear, scaleBand, scaleUtc, scaleOrdinal, scaleLog } from "d3";
import { getStackedIndex } from "../helpers/index.js";
import { COLOR_PALLET } from "../helpers/colorPallet.js";
import { TIME_OFFSET } from "../../constants.js";
import { BandScale, LinearScale, TimeLinearScale, LogScale } from "../scales/index.js";
const useScales = ({
  props,
  originalSeries: series,
  innerHeight,
  innerWidth,
  stackedData,
  currentData,
  scalesStackedDataMap,
  containerRatio
}) => {
  const { xAxis, yAxis, y2Axis, groups: stackedSeries, xScroll, yScroll } = props;
  const getBandDomain = useCallback(
    (axis) => axis.cols ?? [...series[0].data.map((_, i) => i.toString())],
    [series]
  );
  const getPadding = (domainPadding) => typeof domainPadding === "number" ? [domainPadding, domainPadding] : domainPadding ?? [0, 0];
  const getLinearDomain = useCallback(
    (axis, axisString) => {
      const begin = axis?.beginAtZero ? 0 : axis.min ?? void 0;
      const end = axis.max ?? void 0;
      const data = currentData.filter((serie) => serie.scale === axisString || axisString === "x").map(
        (serie) => serie.data.filter((datum) => datum.value !== null).map((datum) => {
          if (typeof datum.value === "number") {
            return datum.value;
          }
          return datum.value[axisString === "y2" ? "y" : axisString];
        })
      );
      const dataFlattened = data.flat();
      const stackedDataFlatten = stackedData.filter((_, i) => scalesStackedDataMap[i] === axisString).flat();
      const minStackedData = min(stackedDataFlatten, (layer) => min(layer, (sequence) => sequence[0]));
      const maxStackedData = max(stackedDataFlatten, (layer) => max(layer, (sequence) => sequence[1]));
      const minData = min(dataFlattened);
      const maxData = max(dataFlattened);
      const types = ["number", "object"];
      const getMin = () => {
        const aux = [];
        if (typeof begin === "number" || typeof begin === "object") aux.push(begin);
        if (types.includes(typeof minData) && minData !== void 0) aux.push(minData);
        if (typeof minStackedData === "number") aux.push(minStackedData);
        return min(aux);
      };
      const getMax = () => {
        const aux = [];
        if (typeof begin === "number" || typeof begin === "object") aux.push(begin);
        if (typeof end === "number" || typeof end === "object") aux.push(end);
        if (types.includes(typeof maxData) && maxData !== void 0) aux.push(maxData);
        if (typeof maxStackedData === "number") aux.push(maxStackedData);
        return max(aux);
      };
      const minRange = getMin();
      const maxRange = getMax();
      const [leftPadding, rightPadding] = getPadding(axis.domainPadding);
      if (typeof minRange === "number" && typeof maxRange === "number") {
        return [
          minRange - (maxRange - minRange) * (leftPadding ?? 0),
          maxRange + (maxRange - minRange) * (rightPadding ?? 0)
        ];
      }
      if (typeof minRange === "object" && typeof maxRange === "object") {
        if (axis.domainPaddingType && TIME_OFFSET[axis.domainPaddingType]) {
          return [
            TIME_OFFSET[axis.domainPaddingType].offset(minRange, -leftPadding),
            TIME_OFFSET[axis.domainPaddingType].offset(maxRange, rightPadding)
          ];
        }
        return [minRange, maxRange];
      }
      return [];
    },
    [currentData, scalesStackedDataMap, stackedData]
  );
  const getScaleTime = (domain, range) => scaleUtc().domain(domain).range(range);
  const getScaleBand = (domain, range, padding = 0.2) => scaleBand().domain(domain).range(range).padding(padding).paddingOuter(0.05);
  const getScaleLinear = (domain, range) => scaleLinear().domain(domain).range(range);
  const getScaleLog = (domain, range) => scaleLog().domain(domain).range(range);
  const scaleFactory = useCallback(
    (axis, axisProps, range = []) => {
      if (!axisProps) return null;
      if (["time", "datetime"].includes(axisProps?.type)) {
        return new TimeLinearScale(getScaleTime(getLinearDomain(axisProps, axis), range), axisProps);
      }
      if (axisProps?.type === "linear" || ["y", "y2"].includes(axis) && axisProps?.type === void 0) {
        return new LinearScale(getScaleLinear(getLinearDomain(axisProps, axis), range), axisProps);
      }
      if (axisProps?.type === "log")
        return new LogScale(getScaleLog(getLinearDomain(axisProps, axis), range), axisProps);
      if (axisProps?.type === "band" || axis === "x" && axisProps?.type === void 0) {
        const realRange = axis === "y" ? range.reverse() : range;
        return new BandScale(getScaleBand(getBandDomain(axisProps), realRange), axisProps);
      }
      return null;
    },
    [getBandDomain, getLinearDomain]
  );
  const xRangeFrom = useMemo(() => containerRatio * (xAxis?.plotRange?.[0] || 0), [containerRatio, xAxis?.plotRange]);
  const xRangeTo = useMemo(
    () => (innerWidth - (xAxis?.plotRange?.[1] || 0)) * (xScroll ? containerRatio : 1),
    [containerRatio, innerWidth, xAxis?.plotRange, xScroll]
  );
  const yRangeTo = useMemo(() => yAxis?.plotRange?.[0] || 0, [yAxis?.plotRange]);
  const yRangeFrom = useMemo(
    () => (innerHeight - (yAxis?.plotRange?.[1] || 0)) * (yScroll ? containerRatio : 1),
    [containerRatio, innerHeight, yAxis?.plotRange, yScroll]
  );
  const y2RangeTo = useMemo(() => y2Axis?.plotRange?.[0] || 0, [y2Axis?.plotRange]);
  const y2RangeFrom = useMemo(
    () => (innerHeight - (y2Axis?.plotRange?.[1] || 0)) * (yScroll ? containerRatio : 1),
    [containerRatio, innerHeight, y2Axis?.plotRange, yScroll]
  );
  const xScale = useMemo(
    () => scaleFactory("x", xAxis, [xRangeFrom, xRangeTo]),
    [scaleFactory, xAxis, xRangeFrom, xRangeTo]
  );
  const yScale = useMemo(
    () => scaleFactory("y", yAxis, [yRangeFrom, yRangeTo]),
    [scaleFactory, yAxis, yRangeFrom, yRangeTo]
  );
  const y2Scale = useMemo(
    () => scaleFactory("y2", y2Axis, [y2RangeFrom, y2RangeTo]),
    [scaleFactory, y2Axis, y2RangeFrom, y2RangeTo]
  );
  const colorScale = useMemo(
    () => scaleOrdinal().domain(series.map((d) => d.name)).range(series.map((d) => COLOR_PALLET[d.color] ?? d.color)).unknown("red"),
    [series]
  );
  const subgroupsWithBars = useMemo(() => {
    const data = currentData.filter(
      (serie) => serie.type === "bar" && !(stackedSeries?.length && getStackedIndex(stackedSeries, serie.name) > -1)
    ).map((serie) => serie.name);
    if (stackedSeries?.length) {
      stackedSeries.forEach((stackedGroup, i) => {
        if (stackedGroup.length > 0) data.push(`stacked-data-${i}`);
      });
    }
    return data;
  }, [currentData, stackedSeries]);
  const getBandwidth = useCallback(() => {
    if (xScale?.type === "BAND") {
      return xScale?.getBandwidth();
    }
    if (yScale?.type === "BAND") {
      return yScale?.getBandwidth();
    }
    const bars = currentData.filter((serie) => serie.type === "bar");
    const barsLength = bars.length;
    if (!barsLength) return 0;
    const maxDataLength = Math.max(...bars.map((bar) => bar.data.length));
    if (xScale && xScale instanceof BandScale) return 0;
    const divisor = Math.max(
      maxDataLength,
      xScale?.scale.ticks().length ?? 0,
      xScale?.getTicks(innerWidth, "").length ?? 0,
      1
    );
    const width = innerWidth / divisor;
    if (containerRatio > 1) {
      return Math.max(0.5, width * containerRatio);
    }
    const modificator = innerWidth > 400 ? 0.5 : 0.8;
    return Math.max(0.5, Math.min(20, Math.trunc(width - width * modificator)));
  }, [currentData, innerWidth, containerRatio, xScale, yScale]);
  const subGroupScale = useMemo(
    () => getBandwidth() ? scaleBand().domain(subgroupsWithBars).range([0, getBandwidth()]).paddingInner(0.55).padding(0.2).paddingOuter(0.01) : null,
    [getBandwidth, subgroupsWithBars]
  );
  return { xScale, y2Scale, yScale, subGroupScale, colorScale, getBandwidth };
};
export {
  useScales
};
//# sourceMappingURL=useScales.js.map
