import * as React from "react";
import { timeFormat, format, utcDay, utcMonth, utcWeek, utcYear } from "d3";
const AXIS_TYPE = {
  BAND: "BAND",
  LINEAR: "LINEAR",
  DATETIME: "DATETIME",
  LOG: "LOG"
};
const getInterval = (interval) => {
  switch (interval) {
    case "DAY":
      return utcDay;
    case "MONTH":
      return utcMonth;
    case "WEEK":
      return utcWeek;
    case "YEAR":
      return utcYear;
    default:
      return null;
  }
};
class Scale {
  scale;
  type;
  constructor(scale) {
    this.type = "";
    this.scale = scale;
  }
  getBandwidth() {
    return 0;
  }
}
class BandScale extends Scale {
  type;
  scale;
  axisProps;
  constructor(scale, axisProps) {
    super(scale);
    this.scale = scale;
    this.type = "BAND";
    this.axisProps = axisProps;
  }
  get(value) {
    if (typeof value !== "string") return 0;
    return this.scale(value);
  }
  getTicks() {
    return this.scale.domain();
  }
  getTickFormatted(t) {
    if (typeof t !== "string") return t;
    if (typeof this.axisProps?.tick?.tickFormat === "function") {
      return this.axisProps?.tick?.tickFormat(t);
    }
    return t;
  }
  getTicksScaled() {
    return this.scale.domain().map((d) => (this.get(d) ?? 0) + this.getBandwidth() / 2);
  }
  getBandwidth() {
    return this.scale.bandwidth();
  }
}
class LinearScale extends Scale {
  type;
  scale;
  axisProps;
  constructor(scale, axisProps) {
    super(scale);
    this.scale = scale;
    this.type = "LINEAR";
    this.axisProps = axisProps;
  }
  get(value) {
    if (typeof value !== "number") return 0;
    return this.scale(value);
  }
  getTickFormatted(t) {
    if (typeof t !== "number") return t.toString();
    if (typeof this.axisProps?.tick?.tickFormat === "function") {
      return this.axisProps?.tick?.tickFormat(t);
    }
    if (typeof this.axisProps?.tick?.tickFormat === "string") {
      return format(this.axisProps?.tick?.tickFormat)(t);
    }
    return t.toString();
  }
  getTicks(dimension) {
    const ticks = this.scale.ticks?.(this.axisProps?.tick?.tickCount ?? Math.ceil(dimension / 80));
    if (this.axisProps?.tick?.tickValues) {
      if (this.axisProps?.tick?.overwriteTicks) return this.axisProps?.tick?.tickValues;
      this.axisProps?.tick?.tickValues.forEach((t) => {
        if (typeof t !== "number") return;
        const isTickAlreadyPresent = ticks.find((tick) => tick === t);
        if (isTickAlreadyPresent) return;
        ticks.push(t);
      });
    }
    return ticks;
  }
  getTicksScaled(dimension) {
    const ticks = this.getTicks(dimension);
    return ticks.map((t) => typeof t === "number" ? this.get(t) : 0);
  }
}
class LogScale extends LinearScale {
  type;
  scale;
  axisProps;
  constructor(scale, axisProps) {
    super(scale, axisProps);
    this.scale = scale;
    this.type = "LOG";
    this.axisProps = axisProps;
  }
  getTickFormatted(t) {
    if (typeof t !== "number") return t.toString();
    if (typeof this.axisProps?.tick?.tickFormat === "function") {
      return this.axisProps?.tick?.tickFormat(t);
    }
    if (typeof this.axisProps?.tick?.tickFormat === "string") {
      return format(this.axisProps?.tick?.tickFormat)(t);
    }
    return format("~s")(t);
  }
}
class TimeLinearScale extends Scale {
  type;
  scale;
  axisProps;
  constructor(scale, axisProps) {
    super(scale);
    this.scale = scale;
    this.type = "DATETIME";
    this.axisProps = axisProps;
  }
  get(value) {
    if (typeof value !== "object") return 0;
    return this.scale(value);
  }
  getTickFormatted(t) {
    if (typeof t !== "object") return t.toString();
    if (typeof this.axisProps?.tick?.tickFormat === "function") {
      return this.axisProps?.tick?.tickFormat(t);
    }
    if (typeof this.axisProps?.tick?.tickFormat === "string") {
      return timeFormat(this.axisProps?.tick?.tickFormat)(t);
    }
    return t.toString();
  }
  getTicks(dimension, intervalType) {
    const interval = getInterval(intervalType);
    const finalTickConfig = interval || (this.axisProps?.tick?.tickCount ?? Math.ceil(dimension / 80));
    const ticks = this.scale.ticks?.(finalTickConfig);
    const ticksValues = this.axisProps?.tick?.tickValues;
    if (ticksValues) {
      if (this.axisProps?.tick?.overwriteTicks) return ticksValues;
      const newTicks = [];
      ticks.forEach((t) => {
        if (typeof t !== "object") return;
        const exists = ticksValues.some(
          (tickValue) => typeof tickValue === "object" && tickValue.getTime() === t.getTime()
        );
        if (!exists) newTicks.push(t);
      });
      return [...newTicks, ...ticksValues];
    }
    return ticks;
  }
  getTicksScaled(dimension, intervalType) {
    const ticks = this.getTicks(dimension, intervalType);
    return ticks.map((t) => typeof t === "object" ? this.get(t) : 0);
  }
}
export {
  AXIS_TYPE,
  BandScale,
  LinearScale,
  LogScale,
  Scale,
  TimeLinearScale
};
//# sourceMappingURL=index.js.map
